#include "eiffel-gtk.h"
#include <gdk/gdktypes.h>
#include <gdk/gdkprivate.h>
#include <gdk/gdk.h>
#include <gtk/gtkwidget.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>             /* memcpy */
#include <limits.h>             /* gulong -> int conversion checks */
#include <time.h>

#ifndef GTK_HAVE_FEATURES_1_1_8
#error Must have at least GTK 1.1.8
#endif

#include "stop.xpm"
#include "exclamation.xpm"
#include "question.xpm"
#include "ok40x40.xpm"
#include "ok20x20.xpm"
#include "cancel40x40.xpm"
#include "cancel20x20.xpm"

/* this is an add-on written by Miron Brezuleanu for the nice wrapper to the
   gtk+ written by P.C. Janzen and F. Penneneach
*/

/* the next functions create GtkPixmaps from the pixmaps above */

struct {
  char *name;
  char **pixmap_data;
} default_pixmaps[] = { { "exclamation", exclamation_xpm },
                        { "question", question_xpm },
                        { "stop", stop_xpm },
                        { "ok40x40", ok40x40_xpm },
                        { "ok20x20", ok20x20_xpm },
                        { "cancel40x40", cancel40x40_xpm },
                        { "cancel20x20", cancel20x20_xpm },
};

GdkPixmap *pixmap;
GdkBitmap *mask;

void
exg_pixmap(GdkWindow *window, char **xpm_data)
{
  GdkColor black;
  black.red = black.green = black.blue = 0;
  pixmap = gdk_pixmap_create_from_xpm_d(window, &mask, &black, xpm_data);
}

int
exg_num_of_pixmaps()
{
  return sizeof(default_pixmaps) / sizeof(default_pixmaps[0]);
}

void
exg_pixmap_by_number(GdkWindow *w, int i)
{
  exg_pixmap(w, default_pixmaps[i].pixmap_data);
}

char *
exg_pixmap_name_by_number(int i)
{
  return default_pixmaps[i].name;
}

GdkPixmap *
exg_current_pixmap()
{
  return pixmap;
}

GdkBitmap *
exg_current_mask()
{
  return mask;
}

/* simple set of GdkColor routines for 16 bit depth */

/* the next one returns the size of GdkColor */

int
ext_sizeof_gdk_color()
{
  return sizeof(GdkColor);
}

/* the next one fills up a GdkColor with needed values; it is the
   reverse of `ext_new_color' */

/* only tested with 16bit depth!!! */

void
ext_new_color_from_pixel(GdkColor *p, int pixel)
{
  GdkVisual *gv = gdk_visual_get_system();
  GdkColormap *cmap;
  switch(gv->depth) {
  case 1:
  case 4: /* I don't know if these are treated right */
  case 8: 
    cmap = gdk_colormap_get_system();
    if (pixel >= cmap->size) {
      printf("ext_new_color_from_pixel: "
	     "color not in colormap; defaulting to black.\n");
      p->pixel = pixel;
      p->red = 0;
      p->green = 0;
      p->blue = 0;
    } else {
      p->pixel = pixel;
      p->red = cmap->colors[pixel].red;
      p->green = cmap->colors[pixel].green;
      p->blue = cmap->colors[pixel].blue;
    }
    break;
  case 15: /* this is highly unlikely */
    p->pixel = pixel;
    p->red = ((pixel >> 10) & 0x1f) << 11;
    p->green = ((pixel >> 5 ) & 0x1f) << 11;
    p->blue = (pixel & 0x1f) << 11;
    break;
  case 16:
    p->pixel = pixel;
    p->red = ((pixel >> 11) & 0x1f) << 11;
    p->green = ((pixel >> 5) & 0x3f) << 10;
    p->blue = (pixel & 0x1f) << 11;
    break;
  case 24:
  case 32:
    p->pixel = pixel;
    p->red = ((pixel >> 16) & 0xff) << 8;
    p->green = ((pixel >> 8) & 0xff) << 8;
    p->blue = (pixel & 0xff) << 8;
    break;
  default:
    printf("Unsupported depth (not 1, 4, 8, 15, 16, 24 or 32 bit)!\n");
    exit(1);
    break;
  }
}

/* the next one fills up a GdkColor with needed values; only tested
   with 16bit depth!!!; This will also adjust the RGB values to match
   the ones returned by `ext_new_color_from_pixel' */

void
ext_new_color(GdkColor *p, int r, int g, int b, GdkColormap *cmap)
{
  GdkVisual *gv = gdk_visual_get_system();
  switch(gv->depth) {
  case 1:
  case 4:
  case 8:
    /* this is for my puny videocard in 1024x768 video mode*/
    p->red = r;
    p->green = g;
    p->blue = b;
    if (!gdk_color_alloc(cmap, p)) {
      printf("unable to allocate color #%2X%2X%2X!\n", r, g, b);
    }
    ext_new_color_from_pixel(p, p->pixel);
    break;
  case 15: /* this is highly unlikely */
    p->red = r;
    p->green = g;
    p->blue = b;
    p->pixel = ((r >> (3+8)) << 10) + ((g >> (3+8)) << 5) + (b >> (3+8));
    ext_new_color_from_pixel(p, p->pixel);
    break;
  case 16:
    p->red = r;
    p->green = g;
    p->blue = b;
    p->pixel = ((r >> (3+8)) << 11) + ((g >> (2+8)) << 5) + (b >> (3+8));
    ext_new_color_from_pixel(p, p->pixel);
    break;
  case 24:
  case 32:
    p->red = r;
    p->green = g;
    p->blue = b;
    p->pixel = ((r >> 8) << 16) + ((g >> 8) << 8) + (b >> 8);
    ext_new_color_from_pixel(p, p->pixel);
    break;
  default:
    printf("Unsupported depth (not 1, 4, 8, 15, 16, 24 or 32 bit)!\n");
    exit(1);
    break;
  }
}

/* the next one retrieves the pixel value */

int
ext_color_pixel(GdkColor *p)
{
  return p->pixel;
}

/* GdkRectangle access fetcher -- once created, these are immutable*/

int
ext_sizeof_gdk_rectangle()
{
  return sizeof(GdkRectangle);
}

void
ext_new_rectangle(GdkRectangle *r, int x, int y, int w, int h)
{
  r->x = x;
  r->y = y;
  r->width = w;
  r->height = h;
}

int
ext_rectangle_x(GdkRectangle *p)
{
  return p->x;
}

int
ext_rectangle_y(GdkRectangle *p)
{
  return p->y;
}

int
ext_rectangle_width(GdkRectangle *p)
{
  return p->width;
}

int
ext_rectangle_height(GdkRectangle *p)
{
  return p->height;
}

/* a function to tell me if this GtkEditable has a selection */

int
ext_editable_has_selection(GtkEditable *e)
{
  return (e->has_selection);
}

/* a couple of functions to retrieve the selection of a GtkEditable */

int
ext_editable_selection_start_pos(GtkEditable *e)
{
  return (e->selection_start_pos);
}

int
ext_editable_selection_end_pos(GtkEditable *e)
{
  return (e->selection_end_pos);
}

/* the next functions are used in EXG_FILE_TOOLS */
/* this should be in another file, but this is it */

/* this a workaround */

int
libc_system(char *s)
{
  return system(s);
}

/* the next function gets the entry subwidget of a combo box */

GtkWidget *
ext_entry_of_combo_box(GtkWidget *combo)
{
  return GTK_COMBO(combo)->entry;
}

/* the next function will set the fields of an adjustment, others than
   value */

void
ext_set_adjustment_fields(GtkWidget *adj,
			  gfloat lower, gfloat upper,
			  gfloat step_increment,
			  gfloat page_increment,
			  gfloat page_size)
{
  GTK_ADJUSTMENT(adj)->lower = lower;
  GTK_ADJUSTMENT(adj)->upper = upper;
  GTK_ADJUSTMENT(adj)->step_increment = step_increment;
  GTK_ADJUSTMENT(adj)->page_increment = page_increment;
  GTK_ADJUSTMENT(adj)->page_size = page_size;
  gtk_signal_emit_by_name(GTK_OBJECT(adj), "changed");  
}

/* the next function is to meant to help EXG_TIME */

void
ext_read_time(int *second, int *minute, int *hour,
	      int *day_of_week, int *day_of_month, int *day_of_year,
	      int *month, int *year)
{
  time_t current_time;
  struct tm *b_time;
  
  time(&current_time);
  b_time = localtime(&current_time);

  if (!b_time) {
    printf("There is something _really_ wrong"
	   " with the time of this machine!\n");
    exit(EXIT_FAILURE);
  }

  *second = b_time->tm_sec;
  *minute = b_time->tm_min;
  *hour = b_time->tm_hour;
  *day_of_week = b_time->tm_wday + 1;
  *day_of_month = b_time->tm_mday;
  *day_of_year = b_time->tm_yday + 1;
  *month = b_time->tm_mon + 1;
  *year = b_time->tm_year + 1900;
}
  
/* for the next function see the comments for `yield' in
   GTK_MAIN.yield */

void
ext_yield()
{
  if (gtk_events_pending())
    gtk_main_iteration();
}

/* to acces the stupid GList stuff without glib and those others
   #@#$@#$*/

int
ext_glist_size()
{
  return sizeof(GList);
}

void
ext_glist_set_data(GList *list, void *data)
{
  list->data = data;
}

void *
ext_glist_get_data(GList *list)
{
  return list->data;
}

void
ext_glist_set_prev(GList *list, void *prev)
{
  list->prev = prev;
}

void *
ext_glist_get_prev(GList *list)
{
  return list->prev;
}

void
ext_glist_set_next(GList *list, void *next)
{
  list->next = next;
}

void *
ext_glist_get_next(GList *list)
{
  return list->next;
}

/* a function to retrieve the parent of a widget */

GtkWidget *
ext_widget_get_parent(GtkWidget *w)
{
  return w->parent;
}

/* to handle Eiffel creation of allocations - needed by
   GTK_WIDGET.absolute_allocation */

/* pretty obvious - to be used by SE's calloc */

int
ext_gtk_allocation_size()
{
  return sizeof(GtkAllocation);
}

/* fills the GtkAllocation structure with the given values */

void
ext_gtk_allocation_fill(GtkAllocation *a, int x, int y, int w, int h)
{
  a->x = x;
  a->y = y;
  a->width = w;
  a->height = h;
}

