indexing
   description: "gdk region"
   status: "You may be able to redistribute and/or modify this program.% 
   % See the README for details."
   copyright: "Copyright (C) 1999 Paul C. Janzen"
   author: "Paul C. Janzen"
   id: "$Id: gdk_rectangle.e,v 1.5 1999/01/18 19:54:55 pcj Exp $"

		 -- actually this is created by me, Miron Brezuleanu, to wrap 
		 -- the region object -- just to compile gdk_gc.e for now, but 
		 -- hopefully I'll complete this wrapper class also

class GDK_REGION
inherit 
   GDK_OBJECT
      redefine 
		 is_equal
      end
creation
   make_default, 
   exg_create,
   make,
   make_as_polygon,
   rect,
   intersection,
   union,
   difference,
   regions_xor
   
feature -- Creation & destruction

   make is 
		 -- just makes a new region
      do
		 gdkobject := ext_region_new
      end

   make_as_polygon(points: COLLECTION[GDK_POINT]; fill_rule: GDK_FILL_RULE) is
		 -- makes the region as a polygon
	  local
		 pts_array: GDK_ARRAY
	  do
		 !!pts_array.from_collection(points)
		 gdkobject := ext_region_polygon(pts_array.to_external, 
										 points.count, fill_rule.value)
	  end

   rect(r: GDK_RECTANGLE) is
		 -- makes the union as a rect
	  local
		 other: POINTER
	  do
		 other := ext_region_new
		 gdkobject := ext_region_union_with_rect(other, r.to_external)
		 ext_region_destroy(other)
	  end

   intersection(r1, r2: GDK_REGION) is
		 -- make a region as the intersection of `r1' and `r2' as pointsets
	  do
		 gdkobject := ext_regions_intersect(r1.to_external, r2.to_external)
	  end   
   
   union(r1, r2: GDK_REGION) is
		 -- make a region as the union of `r1' and `r2' as pointsets
	  do
		 gdkobject := ext_regions_union(r1.to_external, r2.to_external)
	  end   
   
   difference(r1, r2: GDK_REGION) is
		 -- make a region as the difference of `r1' and `r2' as pointsets
	  do
		 gdkobject := ext_regions_substract(r1.to_external, r2.to_external)
	  end   
   
   regions_xor(r1, r2: GDK_REGION) is
		 -- make a region as the eXclusive OR of `r1' and `r2' as pointsets
	  do
		 gdkobject := ext_regions_xor(r1.to_external, r2.to_external)
	  end   
   
   destroy is
		 -- you have to call this by hand for now
      do
		 ext_region_destroy(gdkobject)
      end

   get_clipbox: GDK_RECTANGLE is
		 -- returns the bounding box (I assume)
      do
		 !!Result.make(0, 0, 0, 0)
		 ext_region_get_clipbox(gdkobject, Result.to_external)
      end
   
   empty: BOOLEAN is
		 -- tells whether the region is empty
      do
		 Result := ext_region_empty(gdkobject)
      end

   is_equal(r: like Current): BOOLEAN is
		 -- tells whether the region is empty
      do
		 Result := ext_region_equal(gdkobject, r.to_external)
      end

   point_in(x, y: INTEGER): BOOLEAN is
		 -- is this point in the region ?
      do
		 Result := ext_region_point_in(gdkobject, x, y)
      end

   rect_in(r: GDK_RECTANGLE): BOOLEAN is
		 -- is this rectangle inside the region ?
	  do
		 Result := ext_region_rect_in(gdkobject, r.to_external)
	  end

   offset(dx, dy: INTEGER) is
		 -- it was there; I don't know what this really does
	  do
		 ext_region_offset(gdkobject, dx, dy)
	  end

   shrink(dx, dy: INTEGER) is
		 -- it was there; I don't know what this really does
	  do
		 ext_region_shrink(gdkobject, dx, dy)
	  end


feature -- Commands

feature {NONE} -- Private
   
   ext_region_new: POINTER is
      external "C"
      alias "gdk_region_new"
      end

   ext_region_destroy(p: POINTER) is
      external "C"
      alias "gdk_region_destroy"
      end

   ext_region_get_clipbox(r1, r2: POINTER) is
      external "C"
      alias "gdk_region_get_clipbox"
      end

   ext_region_empty(p: POINTER): BOOLEAN is
      external "C"
      alias "gdk_region_empty"
      end

   ext_region_equal(r1, r2: POINTER): BOOLEAN is
      external "C"
      alias "gdk_region_equal"
      end
   
   ext_region_point_in(p: POINTER; x, y: INTEGER): BOOLEAN is
      external "C"
      alias "gdk_region_point_in"
      end

   ext_region_polygon(p: POINTER; n, fr: INTEGER): POINTER is
	  external "C"
	  alias "gdk_region_polygon"
	  end

   ext_region_rect_in(p1,p2: POINTER): BOOLEAN is
	  external "C"
	  alias "gdk_region_rect_in"
	  end

   ext_region_offset(p: POINTER; i1, i2: INTEGER) is
	  external "C"
	  alias "gdk_region_offset"
	  end

   ext_region_shrink(p: POINTER; i1, i2: INTEGER) is
	  external "C"
	  alias "gdk_region_shrink"
	  end

   ext_region_union_with_rect(p1, p2: POINTER): POINTER is
	  external "C"
	  alias "gdk_region_union_with_rect"
	  end

   ext_regions_intersect(p1, p2: POINTER): POINTER is
	  external "C"
	  alias "gdk_regions_intersect"
	  end

   ext_regions_union(p1, p2: POINTER): POINTER is
	  external "C"
	  alias "gdk_regions_union"
	  end

   ext_regions_substract(p1, p2: POINTER): POINTER is
	  external "C"
	  alias "gdk_regions_substract"
	  end

   ext_regions_xor(p1, p2: POINTER): POINTER is
	  external "C"
	  alias "gdk_regions_xor"
	  end

invariant
end 

