
indexing
   purpose: "define the widget that provides a modified%
   %rubber band selection"
   author: "Miron Brezuleanu"
   
class RS_CANVAS

inherit 
   GTK_DRAWING_AREA
      redefine
	 make,
	 expose_handler,
	 button_1_press_handler,
	 button_1_release_handler,
	 button_2_release_handler,
	 button_3_release_handler,
	 motion_notify_handler,
	 key_release_handler
      end
   EXG_CONSTANTS
   GTK_CONSTANTS
   
creation
   make
   
feature
   
   make is
	 -- sets things up
      do
	 Precursor
	 set_size (200, 200)
	 activate_event_handling
	 selection := Void
	 step := 0
	 dragging := False
      end
   
   draw_background is
	 -- if you want to use this class in your programs, you can 
	 -- just inherit it and redefine this command, which draws 
	 -- the background under the selection
      
	 -- now it draws just a white background
      local
	 gc: GDK_GC
      do
	 !! gc.make (Current)
	 gc.set_foreground (exg_white)
	 draw_rectangle (gc, True, 
			 0, 0, 
			 allocation.width, allocation.height)
	 gc.destroy
      end
   
   expose_handler (area: GDK_RECTANGLE) is
	 -- redraws the canvas
      local
	 gc: GDK_GC
	 r: GDK_RECTANGLE
      do
	 draw_background
	 !! gc.make (Current)
	 inspect step
	 when 0 then
	 when 1 then
	    gc.set_function (gdk_xor)
	    gc.set_foreground (exg_white) 
	    -- `exg_white' is essential for the xor stuff to work!!
	    r := rectangle_from_rubberband
	    draw_rectangle (gc, False,
			    r.x, r.y, r.width, r.height)
	 when 2 then
	 end
	 gc.destroy
      end
   
   button_1_press_handler (eb: GDK_EVENT_BUTTON) is
	 -- handles the creation & modification of the selection
      do
	 inspect step
	 when 0 then
	    dragging := True
	    !! p1.make (eb.int_x, eb.int_y)
	    !! p2.make (eb.int_x, eb.int_y)
	    step := 1
	    expose_handler (Void)
	 when 1 then
	 when 2 then
	    -- for now just restarts the selection
	    step := 0
	    button_1_press_handler (eb) -- nasty kludge
	 end
      end
   
   button_1_release_handler (eb: GDK_EVENT_BUTTON) is
	 -- handles the creation & modification of the selection
      do
	 inspect step
	 when 0 then
	 when 1 then
	    dragging := False
	    step := 2
	 when 2 then
	 end
      end
   
   button_2_release_handler (eb: GDK_EVENT_BUTTON) is
	 -- displays the current selection (if there is one)
      do
	 inspect step
	 when 0 then
	 when 1 then
	 when 2 then
	 end
      end
   
   button_3_release_handler (eb: GDK_EVENT_BUTTON) is
	 -- displays the current selection (if there is one)
      do
	 inspect step
	 when 0 then
	 when 1 then
	 when 2 then
	 end
      end
   
   motion_notify_handler (em: GDK_EVENT_MOTION) is
	 -- handles the creation & modification of the selection
      do
	 inspect step
	 when 0 then
	 when 1 then
	    if dragging then
	       !! p2.make (em.int_x, em.int_y)
	       expose_handler (Void)
	    end
	 when 2 then
	 end
      end
   
   key_release_handler (ek: GDK_EVENT_KEY) is
	 -- handles the creation & modification of the selection
      do
	 inspect step
	 when 0 then
	 when 1 then
	 when 2 then
	 end
      end
   
feature -- queries & attributes
   
   rectangle_from_rubberband: GDK_RECTANGLE is
	 -- will use p1, p2 to give a rectangle as we usually expect 
	 -- one to be (location of top corner, width, height)
      local
	 left, top, width, height: INTEGER
      do
	 left := p1.x.min (p2.x)
	 top := p1.y.min (p2.y)
	 width := p1.x.max (p2.x) - left + 1
	 height := p1.y.max (p2.y) - top + 1
	 !! Result.make (left, top, width, height)
      end
   
   selection: RUBBER_BAND_SELECTION
	 -- the selection object
   
   step: INTEGER
	 -- if 0: we don't have a selection; if 1: we have a normal
	 -- rubber band selection (dragging phase); if 2: we're
	 -- modifying the position of individual points
   
   dragging: BOOLEAN
	 -- `True' if we are dragging something
   
   p1, p2: GDK_POINT
	 -- the corners used during `step=1'
   
end
