
indexing
   author: "Miron Brezuleanu"
   purpose: "utility class to hold all sorts of useful constants; %
   %inherit from it to use them."

class EXG_CONSTANTS

feature -- colors
   
   exg_white: GDK_COLOR is
	 -- the white color as a constant
      once
	 !!Result.make(65535, 65535, 65535)
      end

   exg_black: GDK_COLOR is
	 -- the black color as a constant
      once
	 !!Result.make(0, 0, 0)
      end
   
   exg_red: GDK_COLOR is
	 -- color constants
      once
	 !!Result.make(32767, 0, 0)
      end

   exg_light_red: GDK_COLOR is
	 -- color constants
      once
	 !!Result.make(65535, 0, 0)
      end

   exg_dark_red: GDK_COLOR is
	 -- color constants
      once
	 !!Result.make(16383, 0, 0)
      end

   exg_green: GDK_COLOR is
	 -- color constants
      once
	 !!Result.make(0, 32767, 0)
      end

   exg_light_green: GDK_COLOR is
	 -- color constants
      once
	 !!Result.make(0, 65535, 0)
      end

   exg_dark_green: GDK_COLOR is
	 -- color constants
      once
	 !!Result.make(0, 32767, 0)
      end

   exg_blue: GDK_COLOR is
	 -- color constants
      once
	 !!Result.make(0, 0, 32767)
      end

   exg_light_blue: GDK_COLOR is
	 -- color constants
      once
	 !!Result.make(0, 0, 65535)
      end

   exg_dark_blue: GDK_COLOR is
	 -- color constants
      once
	 !!Result.make(0, 0, 32767)
      end

   best_depth: INTEGER is
	 -- I need the current depth of the display. You bet I'm 
	 -- using the best I can find!
      once
	 Result := ext_visual_get_best_depth
      end

   application_colormap: GDK_COLORMAP is
	 -- if the display is 8bit, this returns the (system's)
	 -- application's colormap, otherwise Void
      once
	 if best_depth > 8 then
	    Result := Void
	 end
	 if best_depth <= 8 then
	    !!Result.make_from_system
	 end
      end
   
   system_visual: GDK_VISUAL is 
	 -- gdk_visual that is the one provided by the system
      once
	 !!Result.exg_create(ext_visual_get_system)
      end
   
   main_window: EXG_MAIN_WINDOW is
      local
	 emw: EXG_MAIN_WINDOW
      once
	 !!emw.make
	 Result := emw.main_window
	 emw.destroy
      end

   exg_default_pixmaps: DICTIONARY[GDK_PIXMAP, STRING] is
	 -- a dictionary containing the exg pixmaps by name
      require
	 main_window.realized
      local
	 i: INTEGER
	 p: GDK_PIXMAP
	 s: STRING
      once
	 !!Result.make
	 from 
	    i := 0
	 until
	    i = exg_num_of_pixmaps
	 loop
	    exg_pixmap_by_number(main_window.window.to_external, i)
	    !!s.from_external_copy(exg_pixmap_name_by_number(i))
	    !!p.create_pixmap(exg_current_pixmap, exg_current_mask)
	    Result.put(p, s)
	    i := i + 1
	 end
      end

   -- some fonts, to unify the look on X11 and Win9x/NT/2000
   -- the size is H1-6, like in HTML
   -- how should I use the ISO8859-2 (latin2) fonts, to get romanian characters?

   exg_h1_font: GDK_FONT is
      once
	 !!Result.make("-adobe-helvetica-medium-r-*-*-24-*-*-*-*-*-*-*")
      end

   exg_h1_bold_font: GDK_FONT is
      once
	 !!Result.make("-adobe-helvetica-bold-r-*-*-24-*-*-*-*-*-*-*")
      end

   exg_h1_it_font: GDK_FONT is
      once
	 !!Result.make("-adobe-helvetica-medium-o-*-*-24-*-*-*-*-*-*-*")
      end

   exg_h1_it_bold_font: GDK_FONT is
      once
	 !!Result.make("-adobe-helvetica-bold-o-*-*-24-*-*-*-*-*-*-*")
      end

   exg_h2_font: GDK_FONT is
      once
	 !!Result.make("-adobe-helvetica-medium-r-*-*-18-*-*-*-*-*-*-*")
      end

   exg_h2_bold_font: GDK_FONT is
      once
	 !!Result.make("-adobe-helvetica-bold-r-*-*-18-*-*-*-*-*-*-*")
      end

   exg_h2_it_font: GDK_FONT is
      once
	 !!Result.make("-adobe-helvetica-medium-o-*-*-18-*-*-*-*-*-*-*")
      end

   exg_h2_it_bold_font: GDK_FONT is
      once
	 !!Result.make("-adobe-helvetica-bold-o-*-*-18-*-*-*-*-*-*-*")
      end

   exg_h3_font: GDK_FONT is
      once
	 !!Result.make("-adobe-helvetica-medium-r-*-*-14-*-*-*-*-*-*-*")
      end

   exg_h3_bold_font: GDK_FONT is
      once
	 !!Result.make("-adobe-helvetica-bold-r-*-*-14-*-*-*-*-*-*-*")
      end

   exg_h3_it_font: GDK_FONT is
      once
	 !!Result.make("-adobe-helvetica-medium-o-*-*-14-*-*-*-*-*-*-*")
      end

   exg_h3_it_bold_font: GDK_FONT is
      once
	 !!Result.make("-adobe-helvetica-bold-o-*-*-14-*-*-*-*-*-*-*")
      end

   exg_h4_font: GDK_FONT is
      once
	 !!Result.make("-adobe-helvetica-medium-r-*-*-12-*-*-*-*-*-*-*")
      end

   exg_h4_bold_font: GDK_FONT is
      once
	 !!Result.make("-adobe-helvetica-bold-r-*-*-12-*-*-*-*-*-*-*")
      end

   exg_h4_it_font: GDK_FONT is
      once
	 !!Result.make("-adobe-helvetica-medium-o-*-*-12-*-*-*-*-*-*-*")
      end

   exg_h4_it_bold_font: GDK_FONT is
      once
	 !!Result.make("-adobe-helvetica-bold-o-*-*-12-*-*-*-*-*-*-*")
      end

   exg_h5_font: GDK_FONT is
      once
	 !!Result.make("-adobe-helvetica-medium-r-*-*-10-*-*-*-*-*-*-*")
      end

   exg_h5_bold_font: GDK_FONT is
      once
	 !!Result.make("-adobe-helvetica-bold-r-*-*-10-*-*-*-*-*-*-*")
      end

   exg_h5_it_font: GDK_FONT is
      once
	 !!Result.make("-adobe-helvetica-medium-o-*-*-10-*-*-*-*-*-*-*")
      end

   exg_h5_it_bold_font: GDK_FONT is
      once
	 !!Result.make("-adobe-helvetica-bold-o-*-*-10-*-*-*-*-*-*-*")
      end

   exg_h6_font: GDK_FONT is
      once
	 !!Result.make("-adobe-helvetica-medium-r-*-*-8-*-*-*-*-*-*-*")
      end

   exg_h6_bold_font: GDK_FONT is
      once
	 !!Result.make("-adobe-helvetica-bold-r-*-*-8-*-*-*-*-*-*-*")
      end

   exg_h6_it_font: GDK_FONT is
      once
	 !!Result.make("-adobe-helvetica-medium-o-*-*-8-*-*-*-*-*-*-*")
      end

   exg_h6_it_bold_font: GDK_FONT is
      once
	 !!Result.make("-adobe-helvetica-bold-o-*-*-8-*-*-*-*-*-*-*")
      end

   exg_default_font: GDK_FONT is
	 -- provides a standard font, which should look good on both X11 
	 -- and MS-Windows
      once
	 Result := exg_h4_font
      end
   
   dialog_stack: EXG_STACK[GTK_WINDOW] is
	 -- a stack of the open modal dialogs (there can be more than 
	 -- one modal active because a modal can open another modal)
      once
	 !!Result.make_stack
      end
   
feature { NONE } -- external stuff
   
   ext_visual_get_system: POINTER is
      external "C"
      alias "gdk_visual_get_system"
      end
   
   ext_visual_get_best_depth:INTEGER is
      external "C"
      alias "gdk_visual_get_best_depth"
      end

   exg_num_of_pixmaps: INTEGER is
      external "C"
      end

   exg_pixmap_by_number(p: POINTER; i: INTEGER) is
      external "C"
      end

   exg_pixmap_name_by_number(i: INTEGER): POINTER is
      external "C"
      end

   exg_current_pixmap: POINTER is
      external "C"
      end

   exg_current_mask: POINTER is
      external "C"
      end

end
