
indexing
   author: "Miron Brezuleanu"
   purpose: "provide a one-call way to retrieve a file name"
   
class EXG_FILE_SELECTION_MODAL
   
inherit
   GTK_MAIN
   GTK_COMMAND
   EXG_CONSTANTS
   
creation
   make
   
feature
   
   make is 
	 -- will initialize the fields
      do
	 answer := clone("cancel")
	 filename := clone("")
	 title := clone("")
	 filesel := Void
      end
   
   set_filename(nv: STRING) is
	 -- will set an initial value for the filename; call this
	 -- before `run'
      do
	 filename := nv
      end
   
   set_title(nv: STRING) is
	 -- will set an initial value for the title; call this before
	 -- `run'
      do
	 title := nv
      end
   
   run is
	 -- will run the fileselection process; when this returns, 
	 -- check `answer' and `filename' for results
      do
	 !!filesel.make(title)
	 filesel.set_filename(filename)
	 filesel.ok_button.signal_connect("clicked", Current, "ok")
	 filesel.cancel_button.signal_connect("clicked", Current, "cancel")
	 filesel.signal_connect("destroy", Current, "destroy")
	 filesel.set_modal(True)
	 filesel.show
	 dialog_stack.push(filesel)
	 from
	    can_exit := False
	 until
	    can_exit
	 loop
	    yield
	 end
	 dialog_stack.pop
      end
   
   execute(d: ANY) is
	 -- will respond to button clicks
      local
	 s: STRING
      do
	 s ?= d
	 check
	    s /= Void
	 end
	 if equal(s, "ok") then
	    filename := clone(filesel.filename)
	    answer := "ok"
	    can_exit := True
	    filesel.destroy
	 elseif equal(s, "cancel") then
	    answer := "cancel"
	    can_exit := True
	    filesel.destroy
	 elseif equal(s, "destroy") and not can_exit then
	    answer := "cancel"
	    can_exit := True
	    filesel.destroy
	 end
      end
   
feature -- attributes
   
   answer: STRING
	 -- will contain the name of the pressed button: "ok" or
	 -- "cancel" (after calling `run')
   
   filename: STRING
	 -- will contain the selected filename, if any (check
	 --`answer'), after calling `run'
   
   title: STRING
	 -- the title of the dialog; defaults to ""
   
feature { NONE } -- private attributes
   
   can_exit: BOOLEAN
	 -- can we return from `run'?
   
   filesel: GTK_FILE_SELECTION
	 -- the actual dialog
   
end
   
