
indexing
   author: "Miron Brezuleanu"
   purpose: "Fill the gaps in the SE -0.78 FILE_TOOLS class;%
   %One should only inherit from this class as it is a facility class"

class EXG_FILE_TOOLS
   
inherit
   FILE_TOOLS

feature
   
   is_directory(path: STRING): BOOLEAN is
	 -- tells whether this is a directory or an ordinary file
	 -- this could be highly optimized as it is very slow now
      local
	 cwd: STRING -- current working directory
	 
      do
	 cwd := current_directory
	 choose_directory(path)
	 if exit_code = 0 then 
	    Result := True
	 end
	 choose_directory(cwd)
      end

   split, split_filename(path: STRING): ARRAY[STRING] is 
	 -- makes a list of STRINGS containing the words separated by
	 -- `\' in the path; for "" returns an empth array, not void
      local
	 path_copy: STRING
      do
	 path_copy := clone(path)
	 path_copy.replace_all('/', ' ')
	 Result := path_copy.split
	 if Result = Void then
	    !!Result.make(1,0)
	 end
      end

   join, join_to_filename(split_path: ARRAY[STRING]): STRING is
	 -- this is the opposite of `split_filename'; if you make a 
	 -- call like `join_to_filename(split_filename(path))' you'll 
	 -- get path, without a `/' at the end
      local
	 i: INTEGER
      do
	 Result := clone("/")
	 from 
	    i := 1
	 until
	    i > split_path.count
	 loop
	    if Result.last = '/' 
	       and split_path.item(i).first = '/' then
	       Result.remove_last(1)
	    end
	    Result := Result + split_path.item(i)
	    if Result.last /= '/' then
	       Result.extend('/')
	    end
	    i := i + 1
	 end
	 Result.remove_last(1)
      end

   relative_path(path_1,path_2: STRING): STRING is
	 -- if given a pair of absolute paths, like `/usr/X11/bin' 
	 -- and `/usr/X11/lib/icewm' it will return the relative 
	 -- path, in this case `../lib/icewm' (that is the path 
	 -- that'll take us to path_2 when path_1 is the working 
	 -- directory; it requires that the paths are really paths 
	 -- and not filenames
      require
	 is_directory(path_1)
	 is_directory(path_2)
      local
	 split_path_1, split_path_2: ARRAY[STRING]
	 i: INTEGER
      do
	 split_path_1 := split_filename(path_1)
	 split_path_2 := split_filename(path_2)
	 from
	 until
	    split_path_1.count = 0 or else split_path_2.count = 0
	       or else not equal(split_path_1.item(1), split_path_2.item(1))
	 loop
	    split_path_1.remove(1)
	    split_path_2.remove(1)
	 end
	 Result := clone("")
	 from
	    i := 1
	 until
	    i > split_path_1.count
	 loop
	    Result := Result + "../"
	    i := i + 1
	 end
	 from 
	    i := 1
	 until
	    i > split_path_2.count
	 loop
	    Result := Result + split_path_2.item(i)
	    Result.extend('/')
	    i := i +1
	 end
      end

   exg_system(command: STRING) is
	 -- this is exactly like GENERAL's system, except it fetches
	 -- the exit code in `exit_code'
      do
	 exit_code := libc_system(command.to_external)
      end

   exit_code: INTEGER

   default_path_len: INTEGER is 1024

   current_directory: STRING is
	 -- gets the working directory 
	 -- this is kludgy, but I couldn't get it to work using the 
	 -- storage space of Result
      local
	 p: POINTER
      do
	 p := libc_getcwd(default_pointer, 0)
	 check
	    not p.is_null
	 end
	 !!Result.from_external_copy(p)
	 libc_free(p)
      end

   choose_directory(path: STRING) is      
	 -- sets the directory; this could be optimized
	 -- exg_sustem_exit_code should be 0 if everything's ok
      do
	 exg_system("cd "+path)
      end

feature {NONE} -- some private stuff

   libc_system(p: POINTER): INTEGER is
	 -- this has to be defined in the C section because the 
	 -- GENERAL's stupid `system' uses up this name!
      external "C"
      end

   libc_getcwd(p: POINTER; i: INTEGER): POINTER is
      external "C use <unistd.h>"
      alias "getcwd"
      end

   libc_free(p: POINTER) is
      external "C"
      alias "free"
      end

end
