
indexing
   author: "Miron Brezuleanu"
   purpose: "takes care of displaying simple messages"

class EXG_MESSAGE_BOX

inherit
   GTK_COMMAND
   GTK_MAIN
   EXG_CONSTANTS
   GTK_CONSTANTS

creation
   make

feature
   
   make is
	 -- inits the message box items with sensible default values
      do
	 set_title("Error")
	 set_message("There is something wrong")
	 set_ok_button(True)
	 set_pixmap(exg_default_pixmaps.at("stop"))
      end

   set_pixmap(ppixmap: GDK_PIXMAP) is
	 -- to ask for another pixmap
      require
	 ppixmap /= Void
      do
	 pixmap := ppixmap
      end

   set_title(ptitle: STRING) is
	 -- sets the title of the message box
      do
	 title := ptitle
      end
	 
   set_message(pmessage: STRING) is
	 -- sets the message of this box
      do
	 message := pmessage
      end
	 
   set_ok_button(flag: BOOLEAN) is
	 -- tells the object if it should have an `ok' button
      do
	 ok_button_flag := flag
      end
	 
   set_cancel_button(flag: BOOLEAN) is
	 -- tells the object if it should have a `cancel' button
      do
	 cancel_button_flag := flag
      end
	 
   set_yes_button(flag: BOOLEAN) is
	 -- tells the object if it should have an `yes' button
      do
	 yes_button_flag := flag
      end
	 
   set_no_button(flag: BOOLEAN) is
	 -- tells the object if it should have an `no' button
      do
	 no_button_flag := flag
      end
	 
   run is
	 -- runs the message box; the result is left as a string in 
	 -- `answer'; it's one of `cancel', `ok', `yes' or `no'
      local
	 hbox, vbox: GTK_BOX
	 button: GTK_BUTTON
	 lbl: GTK_LABEL
	 separator: GTK_SEPARATOR
	 pixmap_widget: GTK_PIXMAP
      do
	 !!window.make_with_type(gtk_window_dialog)
	 window.set_title(title)
	 !GTK_VBOX!vbox.make(False, 2)
	 !GTK_HBOX!hbox.make(False, 2)
	 !!pixmap_widget.make(pixmap)
	 hbox.pack_start(pixmap_widget, False, False, 0)
	 pixmap_widget.show
	 !!lbl.make_with_label(message)
	 lbl.set_alignment (0.0, 0.0)
	 hbox.pack_start(lbl, False, False, 0)
	 lbl.show
	 vbox.pack_start(hbox, False, False, 0)
	 hbox.show
	 !GTK_HSEPARATOR!separator.make
	 vbox.pack_start(separator, False, False, 0)
	 separator.show
	 !GTK_HBOX!hbox.make(True, 2)
	 if ok_button_flag then
	    !!button.make_with_label("Ok")
	    button.signal_connect("clicked", Current, "ok")
	    hbox.pack_start(button, True, True, 0)
	    button.show
	 end
	 if yes_button_flag then
	    !!button.make_with_label("Yes")
	    button.signal_connect("clicked", Current, "yes")
	    hbox.pack_start(button, True, True, 0)
	    button.show
	 end
	 if no_button_flag then
	    !!button.make_with_label("No")
	    button.signal_connect("clicked", Current, "no")
	    hbox.pack_start(button, True, True, 0)
	    button.show
	 end
	 if cancel_button_flag then
	    !!button.make_with_label("Cancel")
	    button.signal_connect("clicked", Current, "cancel")
	    hbox.pack_start(button, True, True, 0)
	    button.show
	 end
	 vbox.pack_start(hbox, True, True, 0)
	 hbox.show
	 window.add(vbox)
	 vbox.show
	 window.signal_connect("destroy", Current, "destroy")
	 window.set_position(gtk_win_pos_center)
	 window.set_modal(True)
	 window.set_policy(False, False, False)
	 window.show
	 dialog_stack.push(window)
	 from
	    can_exit := False
	 until
	    can_exit
	 loop
	    yield
	 end
	 dialog_stack.pop
      end

   execute(d: ANY) is
	 -- handles commands
      local
	 s: STRING
      do
	 s ?= d
	 check
	    s /= Void
	 end
	 if s.is_equal("destroy") and not can_exit then
	    s := "cancel"
	 end
	 if s.is_equal("ok") or s.is_equal("cancel") or 
	    s.is_equal("yes") or s.is_equal("no") then
	    answer := s
	    can_exit := True
	    window.destroy
	 end
      end
   
   ok_button_flag, cancel_button_flag, 
   yes_button_flag, no_button_flag: BOOLEAN
   pixmap: GDK_PIXMAP
   answer: STRING
   title, message: STRING
   
feature {NONE}
   window: GTK_WINDOW
   can_exit: BOOLEAN

end
