#!/usr/bin/env perl

# call this to generate the documentation files; this is one of my
# (Miron) first Perl programs, so it might be ugly and bad. It should
# work, though :)

# thanks to Scott Urbaniak for the suggestion of distributing short
# forms. I hope this script is good enough.

# of course, the hard part is to write useful comments in each .e file
# .... :)

# this first part of the script attempts to extract a description for
# the file passed as argument to `extract_description'.

# the methods used to extract the description (or what qualifies as a
# description):

# the `purpose' and `description' fields of the indexing clause

# the first paragraph (LaTeX meaning) of comments between the class
# declaration and the inherit clause (this is the SmallEiffel style of
# describing the class)

# this one separates the purpose and description strings from the
# indexing clause strings

sub separate_fields {
    my(@indexing) = @_ ;
    my(@description) = () ;
    my(@purpose) = () ;
    my($in_description) = 0 ;
    my($in_purpose) = 0 ;
    my($i) = 0 ;
    for ($i = 0; $i < @indexing; $i = $i + 1) {
	$_ = $indexing[$i] ;
	if (m/^\s*\w+\s*:\s*\"/ || m/^\s*--/) {
	    $in_description = 0 ;
	    $in_purpose = 0 ;
	}
	if (m/^\s*description\s*:\s*\"/) {
	    $in_description = 1;
	    push (@description, $') ;
	    next ;
	}
	if (m/^\s*purpose\s*:\s*\"/) {
	    $in_purpose = 1 ;
	    push (@purpose, $') ;
	    next ;
	}
	if ($in_purpose) {
	    push (@purpose, $_) ;
	}
	if ($in_directory) {
	    push (@directory, $_) ;
	}
    }
    return (\@description, \@purpose) ;
}

# this one strips leading/ending whitespace characters

sub strip_spaces {
    my((@arg)) = @_ ;
    $_ = $arg[0] ;
    if (m/^\s*/) { $_ = $' ; }
    if (m/\s*$/) { $_ = $` ; }
    return $_ ;
}

# this one handles some common Eiffel escape sequences

sub handle_e_escapes {
    my(($line)) = @_ ;
    @line = split (//, $line) ;
    my($result) = "" ;
    my($in_escape) = 0 ;
    foreach (@line) {
	if ($_ eq "%") {
	    $in_escape = 1 ;
	    next ;
	}
	if ($in_escape) {
	    if ($_ eq "%") {
		$result = $result . $_ ;
	    } elsif ($_ eq "\'") {
		$result = $result . $_ ;
	    } elsif ($_ eq "\"") {
		$result = $result . $_ ;
	    }
	    $in_escape = 0 ;
	    next ;
	}
	$result = $result . $_ ;
    }
    return ($result) ;
}

# this one builds a string from Eiffel string pieces

sub build_string {
    my(@epieces) = @_ ;
    my($i) = 0;
    my($result) = "";
    for ($i = 0 ; $i < @epieces ; $i = $i + 1) {
	$_ = $epieces[$i] ;
	$_ = strip_spaces ($_) ;
	if (m/^%/ || m/^\"/) { $_ = $' ; }
	if (m/%$/ || m/\"$/) { $_ = $` ; }
	$_ = strip_spaces ($_) ;
	$_ = handle_e_escapes ($_) ;
	$result = $result . " " . $_ ;	
    }
    return ($result) ;
}

# this one analyzes the indexing contents

sub analyze_indexing {
    my(@indexing) = @_ ;
    my(($description_ref, $purpose_ref)) = separate_fields (@indexing) ;
    my(@description) = @{$description_ref} ;
    my(@purpose) = @{$purpose_ref} ;
    if (@description == () && @purpose == ()) {
	return ("") ;
    }
    my($description) = build_string (@description) ;
    my($purpose) = build_string (@purpose) ;
    my($result) = "";
    ($result = $description)
	|| ($result = $purpose) ;
    return ($result) ;
}

# this one prepares a comment line for analysis by removing unnecesary
# content

sub prepare_comment_line {
    my(($line)) = @_ ;
    if ($line =~ m/--/) { $line = $' ; }
    $line = strip_spaces ($line) ;
    return ($line) ;
}

# this one analyzes the description comments

sub analyze_description {
    my((@description)) = @_ ;
    map ($_ = prepare_comment_line ($_), @description) ;
    my($i) = 0 ;
    my($result) = "" ;
    for ($i = 0 ; $description[$i] eq "" && $i < @description ; $i = $i + 1) {}
    if ($i == @description) { return ("") ; }
    for (; $description[$i] ne "" && $i < @description ; $i = $i + 1) {
	$result = $result . $description[$i] . " " ;
    }
    return ($result) ;
}

# The description is the result
# of extract_description. If there is no description available, then
# the output is "n/a". The description is written on a single line.

sub extract_description {    
    my(($filename)) = @_ ;
    open (INPUT, "<$filename") || die ("can't read $filename!\n") ;
    my($in_indexing) = 0 ;
    my($in_description) = 0 ;
    my(@indexing) = () ;
    my(@description) = () ;
    while (<INPUT>) {
	if (/\s*indexing/) {
	    $in_indexing = 1 ;
	    next ;
	}
	if (/^\s*(?:deferred|expanded|)\s*class/) {
	    $in_indexing = 0 ;
	    $in_description = 1 ;
	    next ;
	}
	if (/^\s*(?:inherit|feature)/) {
	    last ;
	}
	if ($in_description && !m/--/ && !m/^\s*$/) {
	    last ;
	}
	if ($in_indexing) { push (@indexing, $_) ; }
	if ($in_description) { push (@description, $_) ; }
    }
    
    ($description = analyze_indexing(@indexing))
	|| ($description = analyze_description(@description))
	    || ($description = "n/a\n") ;

    return $description ;
}

# end of description guessing part ...

# this one gets a filename and deduces the class name (in lowercase);

sub class_name {
    my(($full_file_name)) = @_ ;
    my($file_name) = (split(m/\//, $full_file_name))[-1] ;
    return substr ($file_name, 0, length($file_name)-2) ;
}

# this one takes care of writing the short forms for the classes in
# one directory; it also registers the class with %classes;

sub write_shorts {
    my(($dir)) = @_ ;
    my(@files) = glob ($dir . "*.e") ;
    foreach $file (@files) {
	my($this_class_name) = class_name ($file) ;
	$command = "short -html2 -short $file >"
	    . $ENV{"EXG"} . "/short/"
		. $this_class_name . ".html" ;
	system ($command) ;
	my($description) = extract_description ($file) ;
	$classes{uc ($this_class_name)} = $description ;
    }
}

# this one deduces the SmallEiffel path from the environment variable;

sub get_se_path {
    my($se_path) = $ENV{"SmallEiffel"} ;
    my(@se_path) = split(m/\//, $se_path) ;
    pop (@se_path) ;
    pop (@se_path) ;
    $se_path = join ("/", @se_path) ;
    return $se_path ;
}

# this implements a simple word wrapping mechanism using underscores
# instead of spaces;

sub wrap_class_name {
    my(($class_name, $word_size)) = @_ ;
    my(@class_name_words) = split(m/_/, $class_name) ;
    my(@result) = (shift (@class_name_words)) ;
    my($in_word) = 0 ;
    my($result_idx) = 0 ;
    
    if (length ($result[0]) > $word_size) {
	unshift (@class_name_words, substr ($result[0], $word_size)) ;
	$result[0] = substr ($result[0], 0, $word_size) ;
	$in_word = 1 ;
    }
    while (@class_name_words > 0) {
	my($next_word) = shift (@class_name_words) ;
	if (length ($result[$result_idx] . "_" . $next_word) > $word_size) {
	    $result_idx = $result_idx + 1 ;
	    if (length ($next_word) > $word_size - !$in_word) {
		$result[$result_idx] = (!$in_word ? "_" : "")
		    . substr ($next_word, 0, $word_size) ;
		unshift (@class_name_words,
			 substr ($next_word, $word_size)) ;
		$in_word = 1 ;
	    } else {
		$result[$result_idx] = (!$in_word ? "_" : "") . $next_word ;
		$in_word = 0 ;
	    }	    
	} else {
	    $result[$result_idx] = $result[$result_idx] . "_" . $next_word ;
	}
    }
    return (@result) ;
}

# this writes the index.html file that should contain a table with all
# the classes in the current repository and a short description of
# them;

# NOTE: all the strings written to this file should be htmlized, but
# they aren't; as a result, index.html may look really weird

sub write_index {
    @index = <DATA> ;
    open (INDEX, ">" . $ENV{"EXG"} . "/short/index.html")
	|| die ("can't write file\n") ;
    foreach (@index) {
	print INDEX ("$_") ;
    }
    print INDEX ("<caption><h2>The Short Form Repository."
		 . " A Big Mess.</h2></caption>\n") ;
    print INDEX ("<tr align=\"center\"><th width=\"30%\">Class Name</th>"
		 . "<th>Class Description</th></tr>") ;
    my(@class_names) = sort (keys (%classes)) ;
    foreach (@class_names) {
	my($href) = lc ($_) . ".html" ;
	if (length ($_) > 20) # I guess this is
	                      # because I don't know enough HTML ...
	{
	    my(@parts) = wrap_class_name ($_, 20) ;
	    my($new_class_name) = join ("<br>", @parts) ;
	    print INDEX ("<tr><td valign=\"top\"><a href=\"$href\">"
			 . "$new_class_name</a></td>"
			 . "<td valign=\"top\">$classes{$_}</td></tr>\n") ;
	} else {
	    print INDEX ("<tr><td valign=\"top\"><a href=\"$href\">"
			 . "$_</a></td>"
			 . "<td valign=\"top\">$classes{$_}</td></tr>\n") ;
	}
    }
    print INDEX ("</table></body></html>") ;
    close (INDEX) ;
}

# this is the start of the main program;

print ("Building documentation from the .sgml files ...\n") ;
system ("make -C doc >/dev/null") ;

%classes = () ;

print ("Generating short form of the classes ...\n") ;
print ("\tEXG classes ...\n") ;
write_shorts ("./callbacks/") ;
write_shorts ("./edgets/") ;
write_shorts ("./eiffel-gtk/gtkwrapper/") ;
write_shorts ("./eiffel-gtk/gtkwrapper/constants/") ;
write_shorts ("./general/") ;
write_shorts ("./imlib/") ;
write_shorts ("./turtle/") ;
write_shorts ("./wizard/") ;

print ("\tSmallEiffel Standard Library classes ...\n") ;
$se_path = get_se_path ;
write_shorts ($se_path . "/lib_iterator/") ;
write_shorts ($se_path . "/lib_number/") ;
write_shorts ($se_path . "/lib_random/") ;
write_shorts ($se_path . "/lib_std/") ;

print ("Writing the index file ...\n");
write_index ;

print ("\nDone. You can start reading at\n") ;
print ("\"" . $ENV{EXG} . "/short/index.html\"\n\n") ;
print ("Happy reading!\n") ;

# the header of index.html is the data included in this file...

__END__

    
<html>
  <head>
    <title>Index of short forms</title>
  </head>
  <body>
    <table width="100%" cols="2" border="1">
    
    
