
indexing
   author: "Miron Brezuleanu"
   purpose: "implement turtle graphics on a GDK_DRAWABLE_WRAPPER"

class EXG_TURTLE

creation
   make

feature -- commands
   
   make(pdrawable: GDK_DRAWABLE_WRAPPER; pgc: GDK_GC) is
	 -- associate with `pdrawable' and `pgc' and initialize data
      do
	 drawable := pdrawable
	 gc := pgc
	 
	 set_heading(90)
	 pen_down
	 set_pos(0, 0)
	 !!stack.make_stack
      end

   pen_down is
	 -- start drawing
      do
	 pen := True
      end

   pen_up is
	 -- stop drawing
      do
	 pen := False
      end
   
   set_heading(nheading: DOUBLE) is
	 -- sets the heading of the turtle
      do
	 heading := nheading / Deg
      end

   head_to(hx, hy: INTEGER) is
	 -- sets the head towards the specified coordinates; not well 
	 -- tested !!!!
      do
	 if (hx -x) = 0 then
	    if hy > y then
	       heading := -Pi / 2
	    else
	       heading := Pi / 2
	    end
	 else
	    heading := ((y-hy).to_double / (hx-x).to_double).atan
	    if hx - x < 0 then
	       heading := heading + Pi
	    end
	 end
      end
   
   set_pos(px, py: INTEGER) is
		 -- sets the position of the turtle
      do
		 x := px
		 y := py
      end

   turn_left(amount: DOUBLE) is
		 -- turns the turtle to the left
      do
		 heading := heading + amount / Deg
      end

   turn_right(amount: DOUBLE) is
		 -- turns the turtle to the right
      do
		 turn_left(-amount)
      end

   forward(amount: INTEGER) is
	 -- the turtle crawls forward
      require
	 drawable /= Void
	 gc /= Void
      local
	 lx, ly: INTEGER
      do
	 lx := x + (amount * heading.cos).rounded
	 ly := y - (amount * heading.sin).rounded
	 if pen then
	    drawable.draw_line(gc, x, y, lx, ly)
	 end
	 x := lx
	 y := ly
      end

   back(amount: INTEGER) is
	 -- the turtle crawls backwards (have you ever seen this in 
	 -- the real world ?)
      do
	 forward(-amount)
      end

   push_status is
		 -- remembers the current position on a stack
      do
	 stack.clone_and_push(Current)
      end

   pop_status is
		 -- recalls the position saved on the stack
      require
	 not stack.empty
      do
	 x := stack.top.x
	 y := stack.top.y
	 heading := stack.top.heading
	 pen := stack.top.pen
	 stack.pop
      end
   

feature -- Data (Queries)

   drawable: GDK_DRAWABLE_WRAPPER
   gc: GDK_GC
   
   x, y: INTEGER
   stack: EXG_STACK[EXG_TURTLE]
   heading: DOUBLE
   pen: BOOLEAN

   gdk_position: GDK_POINT is
	 -- returns the position as a GDK_POINT; this is useful when 
	 -- one wants to use the turtle to generate the points for a 
	 -- polygon which needs filling
      do
	 !!Result.make(x, y)
      end
   
   distance_to(dx, dy: INTEGER): INTEGER is
	 -- returns the distance to the given point
      local
	 lx, ly: INTEGER
      do
	 lx := dx - x
	 ly := dy - y
	 Result := lx * lx + ly * ly
	 Result := Result.to_double.sqrt.rounded
      end
   
end
